/*
 * Created on Jun 10, 2005
 *
 *
 */
package edu.buffalo.nsf.xmlcqa.data.dtd;

/**
 * A class representing Kleene's clousure. The expression of which the closure is
 * being defined can be accessed with {@link #getExpr()} method. The set of words 
 * defined by this regular expression is defined to beexactly those who are 
 * concatenation of any (incuding zero) number of words satisfying the 
 * subexpression.
 *   
 * @author staworko
 */
public class Clousure extends RegExpr
{

    protected RegExpr expr;

    /**
     * Standard constructor. 
     * @param expr the regular expression of which the closure is being created
     */
    public Clousure(RegExpr expr)
    {
        this.expr = expr;
    }

    /**
     * Access the expression of which the closure is defined.
     * @return the regular expression of which the closure is defined by 
     * 		   the given instance 
     */
    public NDFA getExpr()
    {
        return expr;
    }
    
    /**
     * Returns a string representation of this regular expression.
     */
    public String toString()
    {
        return "(" + expr.toString() + ")*";
    }
    
    protected int _getStateCount()
    {
        return expr.getStateCount();
    }

    protected void _fillDelta(boolean[][][] delta, int offset)
    {
        expr._fillDelta(delta, offset);
    }

    protected void _fillEpsilonDelta(boolean[][] delta, int offset)
    {
        int       c = expr.getStateCount();
        boolean[] f = expr.getFinalStates();
        
        for (int i = 0; i < c; i++) {
            if (f[i]) {
                delta[offset + i][offset] = true;
            }
        }

        expr._fillEpsilonDelta(delta, offset);
    }

    protected void _fillFinalStates(boolean[] f, int offset)
    {
        f[offset] 					= true;
        f[offset+getStateCount()-1] = true;
    }
}