
package edu.buffalo.nsf.xmlcqa.data.dtd;

/**
 * A class representing concatenation of two regular expressions. The first and 
 * second subexpressions are accesible by {@link #getExpr1()} and {@link #getExpr2()} 
 * respectively. A concatenation of two regular expressions is satisfied by a word 
 * which is a concatenation of two words satisfying respective regular expressions.  
 *    
 * @author staworko
 */
public class Concat extends RegExpr
{

    protected RegExpr expr1;
    protected RegExpr expr2;

    /**
     * Standard constructor. Does nothing fancy, just initializes the fileds
     */
    public Concat(RegExpr expr1, RegExpr expr2)
    {
        this.expr1 = expr1;
        this.expr2 = expr2;
    }

    /**
     * Returns the first subexpression.
     */
    public NDFA getExpr1()
    {
        return expr1;
    }

    /**
     * Return the second subexpression.
     */
    public NDFA getExpr2()
    {
        return expr2;
    }
    
    /**
     * Returns a string representation of this regular expression. 
     */
    public String toString()
    {
        return "(" + expr1.toString() + ") , (" + expr2.toString() + ")";
    }

    protected int _getStateCount()
    {
        return expr1.getStateCount() + expr2.getStateCount();
    }

    protected void _fillDelta(boolean[][][] delta, int offset)
    {
        int c1 = expr1.getStateCount();
        expr1._fillDelta(delta, offset);
        expr2._fillDelta(delta, offset + c1);
    }

    protected void _fillEpsilonDelta(boolean[][] delta, int offset)
    {
        int       c1  = expr1.getStateCount();
        boolean[] f1 = expr1.getFinalStates();
        
        for (int i = 0; i < c1; i++) {
            if (f1[i]) {
                delta[offset + i][offset + c1] = true;                
            }
        }
        expr1._fillEpsilonDelta(delta, offset);
        expr2._fillEpsilonDelta(delta, offset + c1);
    }

    protected void _fillFinalStates(boolean[] f, int offset)
    {
        expr2._fillFinalStates(f,offset+expr1.getStateCount());
    }
}