/*
 * Created on Feb 23, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package edu.buffalo.nsf.xmlcqa.data.dtd;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Class representing tag label of a document. It's assumed that the number of
 * all tags processed by the system is small enought to be stored in memeory
 * at any time. Therefore we use exactly one object to represent the same
 * labels and comparison of two objects of this class can be done by simple
 * pointer comparison. Also, we identify this object with an integer 
 * {@link #ID idetifier}. Tags can be accessed using two methods: 
 * {@link #get(String tagName)} and {@link #getTag(int tagID)}. The number of 
 * currenty <i>known</i> tags can be obtained with {@link #getCount()} method. 
 * Tags are number with consequitive integers starting with <code>0</code>. It is 
 * important to realize that if new, previously unknown tags are being created 
 * between two calls to the method {@link #getCount()}, then the returned values 
 * will be different. Therefore it is advised to create in advance all tags that 
 * are expected to be used, so that after this process the method 
 * {@link #getCount()} resturns a constant value. 
 * 
 * <p>
 * A tag is also a regular expression specifying only one word: the word 
 * consisting of only the tag's name (accssesible with {@link #getName()}).
 * <p> 
 * 
 * @author staworko
 */
public final class Tag extends RegExpr
{

    /**
     * A counter of currently known tags. It is also used to assign consequtive 
     * identifiers to created Tag instances.   
     */
    private   static int  count = 0;
    
    /**
     * A list of currently known tags. Facilitates fast access to a tag with 
     * a given identifier.   
     */
    protected static List tagList      = new ArrayList();
    
    /**
     * A map between names of already created tags and thier instantions. 
     */
    protected static Map  tags         = new HashMap();
    
    /**
     * A constant tag used to label text nodes in the input document. 
     */
    public    static Tag  PCDATA_TAG   = get("PCDATA");
   
    private static Tag _create(String name)
    {        
        Tag t = new Tag(name,count++);
        
        tags.put(t.name, t);
        tagList.add(t.ID,t);
        
        return t;
    }

    /**
     * A method used to obtain a Tag instance with a given name. If such a tag is 
     * not known, it's being created and stored.  
     */
    public static Tag get(String tagName)
    {
        Tag t = (Tag) tags.get(tagName);
        if (t == null) {
            t = _create(tagName);
        }
        
        return t;
    }

    /**
     * A method used to obtain a Tag of a given identifier. If the <code>tagID</code> is 
     * greater than the number of currently known tags a {@link ArrayIndexOutOfBoundsException} 
     * will be thrown.  
     */
    public static Tag getTag(int tagID)
    {
        return (Tag) tagList.get(tagID);
    }
    
    protected String name;
    protected int    hash;
    
    /**
     * A field storing identifier of the tag. This field is final and therefore is unmodifiable.  
     */
    public final int ID;

    protected Tag(String name, int ID)
    {
        this.ID = ID;
        this.name = name;
        this.hash = _hashCode();
    }

    /**
     * Returns the name of the tag.
     */
    public String getName()
    {
        return name;
    }

    public int _hashCode()
    {
        return name.hashCode();
    }

    /**
     * Returns the hash code value for this tag. This value is equal to 
     * the hash code value of the tag's name. 
     */
    public int hashCode()
    {
        return hash;
    }

    /**
     * Returns the string representation of this resulr expression.
     */
    public String toString()
    {
        return name;
    }

    
    protected int _getStateCount()
    {
        return 2;
    }

    protected void _fillDelta(boolean[][][] delta, int offset)
    {
        delta[ID][offset][offset + 1] = true;
    }

    protected void _fillEpsilonDelta(boolean[][] delta, int offset)
    {
    }

    protected void _fillFinalStates(boolean[] f, int offset)
    {
        f[offset+1] = true;
    }

    public static int getCount()
    {
        return count;
    }

}