/*
 * Created on Sep 24, 2004
 *
 * 
 */
package edu.buffalo.nsf.xmlcqa.data.dtd;

/**
 * A class representing a union of two regular expressions. The first and 
 * second subexpressions are accesible by {@link #getExpr1()} and 
 * {@link #getExpr2()} respectively. A word belongs to a union of regular expressions 
 * if the word belongs to the first subexpresion, second subexpressions, or both of them.  
 * 
 * @author Slawomir Staworko
 */
public class Union extends RegExpr
{

    protected RegExpr expr1;
    protected RegExpr expr2;

    /**
     * Standard constructor. Initializes the object. 
     */
    public Union(RegExpr expr1, RegExpr expr2)
    {
        this.expr1 = expr1;
        this.expr2 = expr2;
    }

    /**
     * Returns the first subexpression.
     */
    public NDFA getExpr1()
    {
        return expr1;
    }

    /**
     * Returns the second subexpression.
     */
    public NDFA getExpr2()
    {
        return expr2;
    }

    /**
     * Returns string representation of the regular expression. 
     */
    public String toString()
    {
        return "(" + expr1.toString() + ") | (" + expr2.toString() + ")";
    }

    protected int _getStateCount()
    {
        return 1 + expr1.getStateCount() + expr2.getStateCount();
    }

    protected void _fillDelta(boolean[][][] delta, int offset)
    {
        expr1._fillDelta(delta, offset + 1);
        expr2._fillDelta(delta, offset + 1 + expr1.getStateCount());
    }

    protected void _fillEpsilonDelta(boolean[][] delta, int offset)
    {
        int c1 = expr1.getStateCount();
        int c2 = expr2.getStateCount();
        
        delta[offset][offset + 1]                     = true;
        delta[offset][offset + c1 + 1] 				  = true;
        expr1._fillEpsilonDelta(delta, offset + 1);
        expr2._fillEpsilonDelta(delta, offset + 1 + c1);
    }

    protected void _fillFinalStates(boolean[] f, int offset)
    {
        expr1._fillFinalStates(f,offset+1);
        expr2._fillFinalStates(f,offset+1+expr1.getStateCount());
    }

}