/*
 * Created on May 31, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.data.query;

import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.tree.TreeNode;


/**
 * @author staworko
 *
 */
public abstract class FilterExpr
{
    public static FilterExpr EMPTY = new FilterExpr() {

        public boolean test(TreeNode node)
        {
            return true;
        }
    };
    
    public static class TagFilter extends FilterExpr{
        private Tag t;
        
        private TagFilter(Tag t)
        {
            this.t = t;
        }
        
        public Tag getTag()
        {
            return t;
        }

        public boolean test(TreeNode node)
        {
            return node.isNode() && node.getTag() == t;
        }
    }

    public static class TextFilter extends FilterExpr{
        private String text;
        
        private TextFilter(String text)
        {
            this.text = text;
        }
        
        public boolean test(TreeNode node)
        {
            return node.isText() && node.getText().equals(text);
        }
    }
    
    
    public static FilterExpr createTagFilter(String name)
    {
        return new TagFilter(Tag.get(name));
    }
    
    public static FilterExpr createTagFilter(Tag t)
    {
        return new TagFilter(t);
    }  

    public static FilterExpr createTextFilter(String text) 
    {
        return new TextFilter(text);
    }

    public abstract boolean test(TreeNode node);

}
