/*
 * Created on Feb 23, 2005
 *
 */
package edu.buffalo.nsf.xmlcqa.data.stream;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Stack;

/**
 * An implementation of {@link edu.buffalo.nsf.xmlcqa.data.stream.TokenSink} 
 * based on a file writer. 
 *  
 * @author staworko 
 */
public class FileTokenSink implements TokenSink
{
    /**
     * Stack contining currently opened tokens. This structure 
     * suppoorts controling if the input token stream is balanced.    
     */
    protected Stack       stack;

    /**
     * A writer to which the tokens are being serialized. 
     */
    protected PrintWriter out;

    
    /**
     * Standard constructor.
     */
    public FileTokenSink(File f) throws IOException
    {
        this.out   = new PrintWriter(new BufferedWriter(new FileWriter(f)));
        this.stack = new Stack();
    }

    /**
     * Standard constructor
     */
    public FileTokenSink(String fileName) throws IOException
    {
        this(new File(fileName));
    }

    public void consume(Token t)
    {
        if (t.isOpening()) {
            out.print("<" + t.getTag().getName() + ">");
            stack.push(t);
        } else if (t.isText()) {
            if (stack.isEmpty()) {
                throw new IllegalArgumentException("The XML document should be closed.");
            }
            out.print(t.getText());
        } else if (t.isClosing()) {
            if (stack.isEmpty()) {
                throw new IllegalArgumentException("The XML document should be closed.");
            }
            Token s = (Token) stack.pop();
            out.print("</" + s.getTag().getName() + ">");
        }
    }

    public void close()
    {
        if (!stack.isEmpty()) {
            throw new IllegalStateException("The XML document is not well-formated");
        }
        out.close();
    }

}