/*
 * Created on Feb 23, 2005
 *
 */
package edu.buffalo.nsf.xmlcqa.data.stream;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.Reader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import com.bea.xml.stream.MXParserFactory;
import edu.buffalo.nsf.xmlcqa.FatalError;

/**
 * Standard implementation of {@link edu.buffalo.nsf.xmlcqa.data.stream.TokenStream}
 * based on a streming parser created with the factory 
 * {@link com.bea.xml.stream.MXParserFactory}.
 * @author staworko 
 */
public class StAXTokenStream implements TokenStream, XMLStreamConstants
{

    protected static XMLInputFactory fac = new MXParserFactory();
    protected XMLStreamReader        f;
    protected Token                  t;
    protected boolean                ignoreErrors;

    /**
     * Standard constructor 
     */
    public StAXTokenStream(File file) throws FileNotFoundException,
            XMLStreamException
    {
        this(file, false);
    }

    /**
     * Standard constructor 
     */
    public StAXTokenStream(File file, boolean ignoreErrors)
            throws FileNotFoundException, XMLStreamException
    {
        this(new BufferedReader(new FileReader(file)), ignoreErrors);
    }

    /**
     * Standard constructor 
     */
    public StAXTokenStream(String fileName) throws FileNotFoundException,
            XMLStreamException
    {
        this(fileName, false);
    }

    /**
     * Standard constructor 
     */
    public StAXTokenStream(String fileName, boolean ignoreErrors)
            throws FileNotFoundException, XMLStreamException
    {
        this(new BufferedReader(new FileReader(fileName)), ignoreErrors);
    }

    /**
     * Standard constructor 
     */
    public StAXTokenStream(Reader in, boolean ignoreErrors)
            throws XMLStreamException
    {
        this(fac.createXMLStreamReader(in), ignoreErrors);
    }

    /**
     * Standard constructor 
     */
    public StAXTokenStream(XMLStreamReader xmlStream, boolean ignoreErrors)
    {
        this.ignoreErrors = ignoreErrors;
        this.f = xmlStream;
        this.t = readNext();
    }

    public boolean hasNext()
    {
        return t != null;
    }

    /**
     * Seeks next toekn on the input stream. Returns 
     * <code>null</code> if the stream has reached its end.  
     */
    protected Token readNext()
    {
        try {
            f.next();
            switch (f.getEventType()) {
            case CHARACTERS:
            case CDATA:
                return Token.createText(f.getText());
            case END_ELEMENT:
                return Token.CLOSING_TOKEN;
            case START_ELEMENT:
                return Token.createTag(f.getLocalName());
            case END_DOCUMENT:
                f.close();
                return null;
            case SPACE:
                return readNext();
            default:
                throw new FatalError("Unknown stream element:"
                        + f.getEventType());
            }
        } catch (XMLStreamException e) {
            if (!ignoreErrors) {
                throw new FatalError(e);
            } else {
                try {
                    close();
                } catch (Throwable t) {
                }; //ignore
                return null;
            }
        }
    }

    public Token next()
    {
        Token temp = t;
        t = readNext();
        return temp;
    }

    public void close()
    {
        try {
            f.close();
        } catch (Throwable t) {
            throw new FatalError(t);
        }
    }
}