/*
 * Created on Oct 13, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.data.stream.generator;

import java.io.IOException;
import java.util.Map;
import java.util.Random;
import edu.buffalo.nsf.xmlcqa.FatalError;
import edu.buffalo.nsf.xmlcqa.data.dtd.Clousure;
import edu.buffalo.nsf.xmlcqa.data.dtd.Concat;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.NDFA;
import edu.buffalo.nsf.xmlcqa.data.dtd.RegExpr;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.data.dtd.Union;
import edu.buffalo.nsf.xmlcqa.data.stream.FileTokenSink;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenSink;


public class RandomDocumentGenerator implements DocumentGenerator
{
    protected DTD dtd;
    protected Tag root;
    protected Random rand;
    
    public RandomDocumentGenerator(DTD dtd, Tag root)
    {
        this.dtd  = dtd;
        this.root = root;
        this.rand = new Random();
    }

    
    long   total;
    double ratio;
    public void generate(String fileName, Map params) throws IOException
    {
        total = ((Long) params.get(SIZE_PARAM)).longValue();        
        ratio = ((Double) params.get(RATIO_PARAM)).doubleValue();
        TokenSink sink = new FileTokenSink(fileName); 
        generate(sink,root,total);
        sink.close();
    }
    
    protected long generate(TokenSink sink, NDFA expr, long size)
    {
        if (expr instanceof Tag) {
            return generateTag(sink,(Tag) expr,size);
        } else if (expr instanceof Clousure) {
            return generateClousure(sink,(Clousure) expr,size);
        } else if (expr instanceof Concat) {
            return generateConcat(sink,(Concat) expr,size);
        } else if (expr instanceof Union) {
            return generateUnion(sink,(Union) expr,size);
        } else if (expr == RegExpr.EPSILON) {
            return generateEpsilon(sink,size);
        } else {
            throw new FatalError("Unhandled regular expression type: " + expr.getClass().getName());
        }
    }

    protected long generateEpsilon(TokenSink sink, long size)
    {
        return 0;
    }

    protected long generateTag(TokenSink sink, Tag tag, long size)
    {
        if (tag == Tag.PCDATA_TAG) {
            genereteText(sink);
            return 1;
        }
        
        long s = 0;
        
        sink.consume(Token.createTag(tag)); 
        s++;
        
        s += generate(sink,dtd.getDefinition(tag),size-1);
        
        sink.consume(Token.CLOSING_TOKEN);
        
        return s;
    }	

    private Token txtTok1 = Token.createText("ANS");
    private Token txtTok2 = Token.createText("PHONNY");
    
    protected void genereteText(TokenSink sink)
    {
        if (rand.nextDouble() < ratio ) {
            sink.consume(txtTok1);
        } else {
            sink.consume(txtTok2);            
        }
    }

    protected long generateUnion(TokenSink sink, Union u, long size)
    {
        return generate(sink,(rand.nextBoolean())?(u.getExpr1()):(u.getExpr2()),size);
    }

    protected long generateConcat(TokenSink sink, Concat concat, long size)
    {
        long size1 = (long) (size * rand.nextGaussian());
        long s = 0;
        
        s += generate(sink,concat.getExpr1(),size1);        
        s += generate(sink,concat.getExpr2(),size - s);
        
        return s;
    }

    protected long generateClousure(TokenSink sink, Clousure c, long size)
    {
        double p = 0.05;
        long s = 0;
        while (size > s) {
            long s1 = (long) (p * rand.nextGaussian() * size);
            
            s += generate(sink,c.getExpr(),s1);            
        }
        
        return s;
    }
}
