/*
 * Created on Mar 6, 2005
 *
 */
package edu.buffalo.nsf.xmlcqa.graph;

import java.util.Collection;
import edu.buffalo.nsf.xmlcqa.Constants;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.NDFA;
import edu.buffalo.nsf.xmlcqa.data.dtd.RegExpr;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.data.query.Query;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenStream;
import edu.buffalo.nsf.xmlcqa.tree.NodeCollector;
import edu.buffalo.nsf.xmlcqa.tree.OperationManager;
import edu.buffalo.nsf.xmlcqa.tree.TreeNode;
import edu.buffalo.nsf.xmlcqa.util.Util;

/**
 * @author staworko
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class CQA implements Constants
{
    protected OperationManager mgr;
    protected int threshold;

    public CQA(OperationManager mgr)
    {
        this(mgr,0);
    }
 
    public CQA(OperationManager mgr, int threshold)
    {
        this.mgr = mgr;
        this.threshold = threshold;
    }
    

    public Collection compute(TokenStream ts, Query q, DTD d)
    {
        Token tok = ts.next();
        TreeNode root = new TreeNode(tok);
        NodeCollector parent = mgr.createRoot(q,root);
        
        //return compute(ts,d,parent).rep; 
        return compute(ts,d,parent).nc.getNodes(q.length - 1);
    }
    
    protected Vertex[] clearState(int depth,Vertex[] x)
    {
        for (int i = 0; i < x.length; i++) {
            x[i] = new Vertex(INF,depth,i);
        }
        
        return x;
    }

    protected Vertex[] startState(NodeCollector parent, Tag t, DTD d )
    {
        Vertex[] start = clearState(0,new Vertex[d.getDefinition(t).getStateCount()]);
        start[0] = new Vertex(0,0,0,parent);
        for (int i = 0; i < start.length; i++) {
            if (d.getDefinition(t).getEpsilonDelta()[0][i]) {
                start[i] = start[0];
            }
        }        
        closure(start,d, d.getDefinition(t));
        return start;
    }

    protected int add(int a, int b)
    {
        return (a == INF || b == INF) ? (INF) : (a + b);
    }

    protected void closure(Vertex[] a, DTD dtd, NDFA def)    
    {
        Tag[][][] ftd = def.getFstTagTrDelta(dtd);
        int[][][] fid = def.getFstIndTrDelta(dtd);
        int[][] mcd = def.getMinCostDelta(dtd);
        boolean[][] epsilonDelta = def.getEpsilonDelta();
        
        for (int i = 0; i < a.length; i++) {
            for (int j = 0; j < a.length; j++) {
                if (!epsilonDelta[i][j]) {
                    int e = add(a[i].c,mcd[i][j]);
                    if (a[j].c == e) {
                        a[j].addIncomingIns(a,dtd,fid[i][j],ftd[i][j]);
                    } else if (a[j].c > e) {
                        a[j].c = e;
                        a[j].dropIncomingEdges();                        
                        a[j].addIncomingIns(a,dtd,fid[i][j],ftd[i][j]);
                    }
                }
            }
        }
    }
  
    protected void consume(NDFA def, DTD dtd, Tag t, NodeCollector child,  
                           Vertex[] curr, Vertex[] next, int r, int d)
    {
        multiply(curr, def.getDelta()[t.ID], r, d, next, child);
        closure(next, dtd, def) ;
    }

    protected void multiply(Vertex[] a, boolean[][] b, int r, int d, Vertex[] c, NodeCollector child)
    {
        for (int i = 0; i < c.length; i++) {
            c[i].c = add(a[i].c, d);
            c[i].addIncomingDel(a[i]);
            for (int j = 0; j < a.length; j++) {
                if (b[j][i]) {
                    int e = add(a[j].c, r);
                    if (c[i].c == e) {
                        c[i].addIncomingRep(a[j],child);
                    } else if (c[i].c > e) {
                        c[i].c = e;
                        c[i].dropIncomingEdges();
                        c[i].addIncomingRep(a[j],child);
                    }
                }
            }
        }
    }

    protected Vertex[] snapshot(Vertex[] curr, Vertex[] mem)
    {
        for (int i = 0; i < curr.length; i++) {
            mem[i] = curr[i];
        }
        
        return mem;
    }
    protected Vertex[] evaluate(Vertex[] next, Vertex[] mem)
    {
        int currDepth = next[0].depth;
        int memDepth = mem[0].depth;
        if (currDepth - memDepth < threshold) {
            return next;
        } else {
            //System.out.println("Registering at depth: " + currDepth);
            return semiForce(next,mem);
        }
    }    

    protected Vertex[] semiForce(Vertex[] next, Vertex[] mem)
    {
        for (int i = 0; i < next.length; i++) {
            next[i].register();
        }
        
        mgr.evaluate(mem);        

        mem = snapshot(next,mem);
        
        for (int i = 0; i < next.length; i++) {
            next[i].unregister();
        }
        
        return next;
    }    

    protected class Result
    {
        int size;
        int rep;
        NodeCollector nc;        

        public Result()
        {
            this(0,0,null);
        }
        
        public Result(int size, int rep, NodeCollector nc)
        {
            this.size = size;
            this.rep = rep;
            this.nc = nc;
        }
    }

    protected Result computeEpsilon(TokenStream ts, NodeCollector parent)
    {
        Result res = new Result();
        res.size = 1;
        res.nc = parent;
        int paren = 1;
        Token tok = null;
        	
        do {
            tok = ts.next();
            switch (tok.getType()) {
            case Token.OPENING:
                res.size++;
                paren++;
            	break;
            case Token.CLOSING:
                paren--;
            	break;
            case Token.TEXT:
                res.size++;
                res.rep++;
            	break;
            }
        } while (paren > 0);
        
        return res;
    }

    protected Result compute(TokenStream ts, DTD dtd, NodeCollector parent)
    {
        TreeNode root = parent.getRoot();
        NDFA def = dtd.getDefinition(root.getTag());
        if (def == RegExpr.EPSILON) {
            return computeEpsilon(ts,parent);
        }
        
        Tag   t = root.getTag();
        Query q = parent.getPath();
        boolean[] finalStates = def.getFinalStates();
        Vertex[] curr = startState(parent,t,dtd);
        Vertex[] mem  = snapshot(curr,new Vertex[def.getStateCount()]);
        Vertex[] next = clearState(1,new Vertex[def.getStateCount()]);
        Vertex[] swap = null;
        Result res = new Result();
        res.size++;
        Token tok = null;
        
        int i = 0;
        while (ts.hasNext() && (tok = ts.next()) != Token.CLOSING_TOKEN) {
            
            NodeCollector child = mgr.createChild(parent,i++,tok);
            Result        ch_res;

            if (tok.isOpening()) {
                ch_res = compute(ts,dtd,child);
                child = ch_res.nc;
            } else {
                ch_res = new Result(1,0,child);
            }
            
            res.size += ch_res.size;

            consume(def, dtd, tok.getTag(), child, curr, next, ch_res.rep, ch_res.size);
            
            swap = curr;
            curr = evaluate(next,mem);
            next = clearState(i+1,swap);
        }
        
        res.rep = scalarMultiplyRep(finalStates,curr);
    
        
        res.nc = mgr.cut(scalarMultiplyCol(res.rep, finalStates, curr));
        
        
        
        return res;
    } 

    protected static int scalarMultiplyRep(boolean[] a, Vertex[] b)
    {
        int result = INF;
        
        for (int i = 0; i < a.length; i++) {
            if (a[i] && result > b[i].c) {                
                result = b[i].c;
            }
        }
        
        return result;
    }

    protected NodeCollector[] scalarMultiplyCol(int min, boolean[] a, Vertex[] b)
    {
        NodeCollector[] result = new NodeCollector[0];
        
        for (int i = 0; i < a.length; i++) {
            if (a[i] && b[i].c == min) {
                b[i].register();
            } 
        }
        

        for (int i = 0; i < a.length; i++) {
            if (a[i] && b[i].c == min) {
                mgr.evaluate(b[i]);
                result = Util.concat(result,b[i].getValue());
            }
        }
        
        return result;
    }

}