/*
 * Created on Nov 6, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.graph;

import edu.buffalo.nsf.xmlcqa.Constants;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.NDFA;
import edu.buffalo.nsf.xmlcqa.data.dtd.RegExpr;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenStream;


/**
 * @author staworko
 *
 */
public class DIMRestaurationAutomaton extends RestaurationAutomaton
{
    
    protected static class Result 
    {
        int   rep;
        int   del;
        int[] mod;
        
        Result(int rep, int del, int count) 
        {
            this.rep = rep;
            this.del = del;
            this.mod = new int[count];
            for (int i = 0; i < count; i++) {
                mod[i]= 1;
            }
        }
    }
    
    public static int dist(TokenStream ts, DTD d)
    {
        Tag t = ts.next().getTag();
        
        return dist(ts,d,t).rep;
    }
    
    protected static Result dist(TokenStream ts, DTD d, Tag t)
    {
        int n = Tag.getCount();       
        NDFA[] def = new NDFA[n];
        boolean[][] finalStates = new boolean[n][]; 
        int[][] curr = new int[n][];
        int[][] next = new int[n][];
        int[][] swap = new int[n][];
        Result res = new Result(0,1,n);
        
        for (int i = 0; i < n; i++) {            
            finalStates[i] = def[i].getFinalStates();
            curr[i] = startState(def[i],d);
            next[i] = new int[def[i].getStateCount()];
            swap[i] = null;            
        }
        
        Token tok = null;
        
        while (ts.hasNext() && (tok = ts.next()) != Token.CLOSING_TOKEN ) {
            if (tok.isText()) {
                //handle text nodes
            }
            
            Result child = dist(ts,d,tok.getTag());
            for (int i = 0; i < n; i++) {
                next[i] = clearState(next[i]);
                consume(def[i],d,tok.getTag(),curr[i],next[i],child);
            }
            res.del += child.del;
            swap = curr;
            curr = next;
            next = swap;
        }
        
        for (int i = 0; i < n; i++) {
            if (i == t.ID) {                
                res.rep += scalarMultiply(finalStates[i],curr[i]);
                res.mod[i] = INF;
            } else {
                res.mod[i] += scalarMultiply(finalStates[i],curr[i]);
            }
        }
        
        return res;
    }

    private static void consume(NDFA def, DTD d, Tag tag, 
                                int[] curr, int[] next, Result child)
    {
        for (int i = 0 ; i < next.length; i++) {
            next[i] = add(curr[i],child.del);
            for (int t = 0; t < Tag.getCount(); t++) {
                for (int j = 0; j < next.length; j++) {
                    if (def.getDelta()[t][j][i]) {
                        int step = (tag.ID == t)? (add(curr[j],child.rep)) : (add(curr[j],child.mod[t]));
                        if (step < next[i]) {
                            next[i] = step;
                        }
                    }
                }
            }
        }
        
        multiply(next, def.getMinCostDelta(d), next);        
    }
    
}
