/*
 * Created on Jun 4, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.graph;

import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.tree.NodeCollector;


public final class Edge
{
    public static final int INS = 1;
    public static final int DEL = 2;
    public static final int REP = 3;
    public static final int MOD = 4;
    
    public int           op;
    
    /*insert*/
    public Tag           t;
    public DTD           dtd;
    
    /*repair*/
    public NodeCollector child;

    /* modify */
    
    
    public  Edge   nextIn;
    public  Edge   nextOut;
    public  Vertex from;
    public  Vertex to;
    
    public  boolean           registered = false;
    public  Edge              domain     = null;
    
    public Edge(Vertex from, Vertex to)
    {   
        this.from = from;
        this.to = to;
    }

    public Edge(Vertex from, Tag t, DTD dtd, Vertex to, Edge nextIn)
    {
        this(from,to);
        this.op = INS;        
        this.t = t;
        this.dtd = dtd;
        this.nextIn = nextIn;
    }

    public Edge(Vertex from, Vertex to, Edge nextIn) 
    {
        this(from,to);
        this.op = DEL;
        this.nextIn = nextIn;
    }

    public Edge(Vertex from, NodeCollector child, Vertex to, Edge nextIn)
    {
        this(from,to);
        this.op = REP;
        this.child = child;
        this.nextIn = nextIn;
    }

    public void register()
    {
        if (!registered) {
            registered = true;
            this.domain = from.registerOutgoingEdge(this);
        }
    }
    
    public boolean willCauseCloning()
    {
        return domain.from.willCauseCloning() 
        	|| domain.from.willClone();
    }
    
    public void unregister() 
    {
        if (registered) {
            domain     = null;
            registered = false;
            from.unregister();
        }
    }
    
    
    private static String[] op2str={"NOP","INS","DEL","REP"};
    
    public String toString()
    {
        return "E" + ((registered)?(""):("r")) + "[" + "op=" + op2str[op] + "; from = " + from  + "]";
    }
    
}