/*
 * Created on May 12, 2005
 *
 */
package edu.buffalo.nsf.xmlcqa.graph;

import edu.buffalo.nsf.xmlcqa.Constants;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.NDFA;
import edu.buffalo.nsf.xmlcqa.data.dtd.RegExpr;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenStream;

/**
 * @author staworko
 *  
 */
public class RestaurationAutomaton implements Constants
{
    protected static final int REP  = 0;
    protected static final int DEL  = 1;
    protected static final int SIZE = 1;

    public static int dist(TokenStream ts, DTD d)
    {
        return dist(ts, d, ts.next().getTag())[REP];
    }

    protected static int[] clearState(int[] x)
    {
        for (int i = 0; i < x.length; i++) {
            x[i] = INF;
        }
        return x;
    }

    protected static int[] startState(NDFA def, DTD d)
    {
        int[] start = clearState(new int[def.getStateCount()]);
        start[0] = 0;
        multiply(start, def.getMinCostDelta(d), start);
        return start;
    }

    private static int[] distEmpty(TokenStream ts, Tag root)
    {
        int[] res = new int[2];
        res[SIZE] = 1;
        int paren = 1;
        Token t = null;
        do {
            t = ts.next();
            res[SIZE]++;
            switch (t.getType()) {
            case Token.OPENING:
                paren++;
                break;
            case Token.CLOSING:
                paren--;
                break;
            case Token.TEXT:
                res[REP]++;
            }
        } while (paren > 0);
        return res;
    }

    private static int[] dist(TokenStream ts, DTD dtd, Tag root)
    {
        NDFA def = dtd.getDefinition(root);
        if (def == RegExpr.EPSILON) {
            return distEmpty(ts, root);
        }
        boolean[] finalStates = def.getFinalStates();
        int[] curr = startState(def, dtd);
        int[] next = clearState(new int[def.getStateCount()]);
        int[] swap = null;
        int[] res = new int[2];
        res[SIZE] = 1;
        Token t = null;
        while (ts.hasNext() && (t = ts.next()) != Token.CLOSING_TOKEN) {
            if (t.isText()) {
                if (root != Tag.PCDATA_TAG) {
                    for (int i = 0; i < curr.length; i++) {
                        curr[i]++;
                        res[DEL]++;
                    }
                }
                continue;
            }
            //Opening tag
            int[] child = dist(ts, dtd, t.getTag());
            consume(def, dtd, t.getTag(), curr, next, child[REP], child[DEL]);
            res[DEL] += child[SIZE];
            swap = curr;
            curr = next;
            next = clearState(swap);
        }
        res[REP] = scalarMultiply(finalStates, curr);
        return res;
    }

    protected static void consume(NDFA def, DTD dtd, Tag t, int[] curr,
                                  int[] next, int r, int d)
    {
        multiply(curr, def.getDelta()[t.ID], r, d, next);
        multiply(next, def.getMinCostDelta(dtd), next);
    }

    protected static void multiply(int[] a, boolean[][] b, int r, int d, int[] c)
    {
        for (int i = 0; i < c.length; i++) {
            c[i] = add(a[i], d);
            for (int j = 0; j < a.length; j++) {
                if (b[j][i] && c[i] > add(a[j], r)) {
                    c[i] = a[j] + r;
                }
            }
        }
    }

    protected static int add(int a, int b)
    {
        return (a == INF || b == INF) ? (INF) : (a + b);
    }

    protected static void multiply(int[] a, int[][] b, int[] c)
    {
        for (int i = 0; i < c.length; i++) {
            for (int j = 0; j < a.length; j++) {
                if (c[i] > add(a[j], b[j][i])) {
                    c[i] = a[j] + b[j][i];
                }
            }
        }
    }
    
    protected static int scalarMultiply(boolean[] a, int[] b)
    {
        int result = INF;
        
        for (int i = 0; i < a.length; i++) {
            if (a[i] && result > b[i]) {                
                result = b[i];
            }
        }
        
        return result;
    }
}