/*
 * Created on Jun 4, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.graph;

import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.tree.NodeCollector;
import edu.buffalo.nsf.xmlcqa.tree.OperationManager;
import edu.buffalo.nsf.xmlcqa.util.Profiler;


public final class Vertex
{
    public  int  			depth;
    public  int             pos;
    public  int  			c;
    public  Edge 			in = null;    
    public  Edge 			out = null;

    private boolean    		registered       = false;
    public Vertex   		domain           = null;
    public Vertex           nextDom          = null;

    private int     		uses             = 0;
    private Edge     		repDomain        = null;
    private boolean  		willClone        = false;
    
    private boolean         cloningEst       = false;
    private boolean         willCauseCloning = false;
    
    public  NodeCollector[] value            = null;
    

    public Vertex(int c, int depth, int pos)
    {
        super();
        this.c = c;
        this.value = null;
        this.depth = depth;
        this.pos = pos;
    }
    
    public Vertex(int c, int depth, int pos, NodeCollector val)
    {
        this(c,depth,pos);
        
        value = new NodeCollector[1];
        value[0] = val;
    }

    public void addIncomingIns(Vertex from, DTD dtd, Tag t)
    {
        this.in = new Edge(from,t,dtd,this,this.in);
    }
    
    public void addIncomingIns(Vertex[] from, DTD dtd, int[] is, Tag[] tags)
    {
        for (int i = 0; i < is.length; i++) {
            addIncomingIns(from[is[i]],dtd,tags[i]);
        }
    }    
    
    public void addIncomingRep(Vertex from, NodeCollector child)
    {
        this.in = new Edge(from,child,this,this.in);
    }
    
    public void addIncomingDel(Vertex from)
    {
        this.in = new Edge(from,this,this.in);
    }
    
    public void addToDomain(Vertex v)
    {
        v.nextDom = nextDom;
        nextDom = v;
    }
    
    public void addOutgoingEdge(Edge e)
    {
        e.nextOut = out;
        out = e;
    }
    
    public void dropIncomingEdges()
    {
        in  = null;
    }
    
    public void dropOutgoingEdges()
    {
        out = null;
    }
    
    public boolean containsIsoInput(Edge e)
    {
        Edge tmp = in;
        while (tmp!= null) {
            if (e.domain == tmp.domain) {
                return true;
            }
            tmp = tmp.nextIn;
        }
        
        return false;
    }
    
    public boolean containsIsoOutput(Edge e)
    {
        Edge tmp = out;
        while (tmp!= null) {
            if (e.domain == tmp.domain) {
                return true;
            }
            tmp = tmp.nextOut;
        }
        
        return false;
        
    }
    
    public void    removeFollowingIsoDuplicates(Edge e)
    {
        Edge curr = e;
        while (curr.nextIn != null) {
            if (curr.nextIn.domain == e.domain) {                
                curr.nextIn = curr.nextIn.nextIn;
            } else {
                curr = curr.nextIn;
            }
        }
    }
    

    public boolean isRegistered()
    {
        return registered;
    }
    
    public void register()
    {
        Profiler.start("Vertex.register()");
        if (! registered) { 
            registered = true;
            if (!isEvaluated()) {
                registerEdges();
            }
            findDomain();
        }       
        Profiler.stop();
    }
    
    public void registerEdges()
    {
        Profiler.start("Vertex.registerEdges()");
        Edge e = in;
        while (e != null) {
            e.register();
            e = e.nextIn;            
        }   
        Profiler.stop();
    }    
    private void findDomain()
    {
        Profiler.start("Vertex.findDomain()");
        if (domain != null) {
            Profiler.stop();
            return;
        };
        if (in == null) {
            domain = this;
            Profiler.stop();
            return;
        }
        
        Vertex dc = in.domain.to;
        Edge e = in.domain;
        while (e != null) {
            if (!dc.containsIsoInput(e)) {
                domain = this;
                Profiler.stop();
                return;
            }
            e = e.nextIn;
        }
        
        domain = dc;
        if (domain != this) {
            domain.addToDomain(this);
        }
        Profiler.stop();
    }
    
    public Edge registerOutgoingEdge(Edge e)
    {
        Profiler.start("Vertex.registerOutgoingEdge()");
        register();
        
        addOutgoingEdge(e);
        
        return (Edge) Profiler.stop(domain.registerUse(e));
    }
    
    public Edge registerUse(Edge e)
    {
        Profiler.start("Vertex.registerUse()");       
        uses++; 
        if (e.op == Edge.REP) {
            if (repDomain == null) {
                repDomain = e;
            } 
            Profiler.stop();
            return repDomain;
        } else {
            willClone = true;
            Profiler.stop();
            return e;
        }
    }
    public void unregister()
    {
        Profiler.start("Vertex.unregister()");
        if (registered) {
            registered   = false;
            uses         = 0;
            repDomain    = null;
            willClone    = false;
            cloningEst   = false;
            dropOutgoingEdges();
            
            if (!isEvaluated()) {
                unregisterEdges();
                domain       = null;
                nextDom      = null;
            }
        }
        
        Profiler.stop();
    }
    
    public void unregisterEdges()
    {
        Profiler.start("Vertex.unregisterEdges()");
        
        Edge e = in;
        while (e != null) {
            e.unregister();
            e = e.nextIn;            
        }
        
        Profiler.stop();
    }
    public boolean isEvaluated()
    {
        return value != null;
    }
    
    public boolean willCauseCloning()
    {
        Profiler.start("Vertex.willCauseCloning()");
        boolean result = domain._willCauseCloning();
        Profiler.stop();
        return result;
    }
    
    public boolean _willCauseCloning()
    {
        if (cloningEst) {
            return willCauseCloning && !isEvaluated();
        }
        
        cloningEst = true;
        willCauseCloning = false;
        
        Edge e = in;
        while (e != null && !willCauseCloning) {
            willCauseCloning = willCauseCloning || e.willCauseCloning();
            e = e.nextIn;
        }
        
        return willCauseCloning;
    }    
    
    public boolean willClone()
    {
        return domain.willClone;
    }
    
    public NodeCollector[] useValue(OperationManager mgr)
    {
        if (!isEvaluated()) {
            throw new IllegalStateException("The vertex doesn't have a value."); 
        }
        
        return domain._useValue(mgr);
    }
    
    private NodeCollector[] _useValue(OperationManager mgr)
    {
        if (!willClone) {
            return value;
        } else {
            if (uses == 0) {
                throw new IllegalArgumentException("Unregistered use.");                
            }
            return mgr.clone(value);
        }
    }
    
    public NodeCollector[] getValue()
    {
        if (!isEvaluated()) {
            throw new IllegalStateException("The vertex doesn't have a value."); 
        }
        
        return value;
    }
 
 
    public void setValue(NodeCollector[] value)
    {
        if (isEvaluated()) {
            throw new IllegalStateException("This vertex already has a value.");
        }
        
        this.value = value;
    }

    public String toString()
    {
        return "V"+((registered)?("r"):("")) + ((value!= null)?("*"):("!"))+"(depth=" + depth + ", pos=" + pos + ", c=" + c + ")"; 
    }  
}
