package edu.buffalo.nsf.xmlcqa.test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import edu.buffalo.nsf.xmlcqa.Constants;
import edu.buffalo.nsf.xmlcqa.data.dtd.Clousure;
import edu.buffalo.nsf.xmlcqa.data.dtd.Concat;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.NDFA;
import edu.buffalo.nsf.xmlcqa.data.dtd.RegExpr;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.data.dtd.Union;
import edu.buffalo.nsf.xmlcqa.data.stream.FileTokenSink;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;

public class MaxDocSize {
	
	private Random r;
	
	private FileTokenSink f;
	
	private List tokenList = new ArrayList();
	
	public MaxDocSize(String filename) {
		super();
		r = new Random();
		try {
			f = new FileTokenSink(filename);
		} catch (IOException io) {
			System.out.println(io.getMessage());
		}
		// TODO Auto-generated constructor stub
	}
	
	public int maxDoc(NDFA e, DTD d) {
		
		if (e instanceof Clousure) {
			return (Constants.INF);
		}
		
		if (e instanceof Union) {
			Union u = (Union) e;
			return (max(maxDoc(u.getExpr1(), d), maxDoc(u.getExpr2(), d)));
		}
		
		if (e instanceof Concat) {
			Concat c = (Concat) e;
			return (maxDoc(c.getExpr1(), d) + maxDoc(c.getExpr2(), d));
		}
		
		if (e instanceof Tag) {
			Tag t = (Tag) e;
			return (d.getMaxTreeSize(t));
		}
		
		return (0);
	}
	
	public int minDoc(NDFA e, DTD d) {
		
		if (e instanceof Clousure) {
			NDFA e2 = ((Clousure) e).getExpr();
			return (minDoc(e2, d));
		}
		
		if (e instanceof Union) {
			Union u = (Union) e;
			int m1 = minDoc(u.getExpr1(), d);
			int m2 = minDoc(u.getExpr2(), d);
			return (m1 < m2) ? m1 : m2;
		}
		
		if (e instanceof Concat) {
			Concat c = (Concat) e;
			return (minDoc(c.getExpr1(), d) + minDoc(c.getExpr2(), d));
		}
		
		if (e instanceof Tag) {
			Tag t = (Tag) e;
			return (d.getMinTreeSize(t));
		}
		
		return (0);
	}
	
	public int genDoc(NDFA e, int size, DTD d) {
		
		if (e instanceof Clousure) {
			int s = 0;
			
			Clousure c = (Clousure) e;
			double l;
			int size1 = size;

			int min = minDoc(c.getExpr(), d);
			int max = maxDoc(c.getExpr(), d);
			
			while (size > 0) {
				l = r.nextGaussian();
				
				if (max <= min) {
					s = max;
				} else {
					// a random number number between max and min
					s = (int) ((l * (max - min)) + min);
				}
				
				s = genDoc(c.getExpr(), s, d);
				size -= s;
			} // end while size>0
			
			return (size1 - size);
		}
		
		/*
		 * The regular expression is a union of 2 expressions
		 */
		if (e instanceof Union) {
			Union u = (Union) e;
			NDFA n = u.getExpr1();
			int s;
			
			/*
			 * randomly select either of the 2 union expressions
			 */
			if (r.nextBoolean()) {
				n = u.getExpr2();
			}
			s = genDoc(n, size, d);
			return (s);
		}
		
		if (e instanceof Concat) {
			Concat c = (Concat) e;
			int s, max1, min1, max2, min2, s1, s2;
			
			double l = r.nextGaussian();
			
			// calculate size for expression 1
			NDFA e1 = c.getExpr1();
			NDFA e2 = c.getExpr2();
			
			/*
			 * randomly order the 2 expressions
			 */
			if (r.nextBoolean()) {
				e1 = c.getExpr2();
				e2 = c.getExpr1();
			}
			max1 = maxDoc(e1, d);
			min1 = minDoc(e1, d);
			
			max2 = maxDoc(e2, d);
			min2 = minDoc(e2, d);
			
			if (max1 <= min1) {
				s1 = max1;
			} else {
				// a random number number between max and min
				s1 = (int) ((l * (max1 - min1)) + min1);
			}
			
			// calculate size for expression 2
			s2 = size - s1;
			if (s2 > max2) {
				
				s2 = max2;
				s1 = size - s2;
			}
			
			s = genDoc(e1, s1, d) + genDoc(e2, s2, d);
			return (s);
		}
		
		if (e instanceof Tag) {
			Tag t = (Tag) e;
			// the size of document generated
			int s = 0;
			
			if (t == Tag.PCDATA_TAG) {
				f.consume(Token.createText("Data"));
				s = 1;
			} else if (t == Tag.EPSILON) {
				System.out.println("empty");
			} else {
				
				// recursive call for the regular expression specifying the
				// children of the tag
				
				f.consume(Token.createTag(t));
				s = genDoc(d.getDefinition(t), size - 1, d);
				f.consume(Token.CLOSING_TOKEN);
			}
			
			return (s);
		}
		return (0);
	}	

    public static int max(int i, int j) {
		
		return (i > j ? i : j);
	}
	
	public static int calculateSize(NDFA e, int size) {
		return (10);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		
		MaxDocSize m = new MaxDocSize("test.xml");
		
		Tag a = Tag.get("a");
		Tag b = Tag.get("b");
		Tag c = Tag.get("c");
		Tag d = Tag.get("d");
		
		Map defs = new HashMap(4);
		defs.put(a, RegExpr.EPSILON);
		defs.put(b, Tag.PCDATA_TAG);
		defs.put(c, Tag.PCDATA_TAG);
		defs.put(d, new Clousure(new Union(new Concat(a, b), c)));
		
		DTD dtd = new DTD(defs);
		
		// System.out.println(Tag.getCount());
		for (int i = 0; i < Tag.getCount(); i++) {
			m.tokenList.add(i, Token.createTag(i));
		}
		// RegExpr.EPSILON);
		// RegExpr r = (new Union(new Clousure(new Concat(a, b)),
		// Tag.PCDATA_TAG));
		
		// System.out.println(r);
		// System.out.println(m.minDoc(c, DTDs.dtd[0]));
		System.out.println(dtd);
		System.out.println(m.genDoc(Tag.get("d"), 1000000, dtd));
		System.out.println("Done");
		m.f.close();
		
	}
	
}
