/*
 * Created on May 12, 2005
 *
 */
package edu.buffalo.nsf.xmlcqa.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.xml.stream.XMLStreamException;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.ValidationAutomaton;
import edu.buffalo.nsf.xmlcqa.data.stream.FileTokenSink;
import edu.buffalo.nsf.xmlcqa.data.stream.StAXTokenStream;
import edu.buffalo.nsf.xmlcqa.data.stream.StreamDist;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenSink;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenStream;
import edu.buffalo.nsf.xmlcqa.graph.RestaurationAutomaton;

/**
 * @author staworko
 *  
 */
public class Validation_vs_Restauration
{

    private static DTD        dtd  = DTDs.dtd[0].init();
    private static DateFormat form = new SimpleDateFormat("ssssss.SSS");
    private static Token      A    = Token.createTag("a");
    private static Token      B    = Token.createTag("b");
    private static Token      C    = Token.createTag("c");

    public static void readFile(String file) throws IOException
    {
        Reader r = new BufferedReader(new FileReader(file));
        while (r.read() != -1) {
        }
        r.close();
    }

    public static void readStream(TokenStream ts)
    {
        while (ts.hasNext()) {
            Token t = ts.next();
        }
    }

    public static void generateFile(String file, long size) throws IOException
    {
        TokenSink sink = new FileTokenSink(file);
        sink.consume(C);
        for (long i = 0; i < size; i++) {
            sink.consume(A);
            sink.consume(Token.CLOSING_TOKEN);
            sink.consume(B);
            sink.consume(Token.CLOSING_TOKEN);
        }
        sink.consume(Token.CLOSING_TOKEN);
        sink.close();
    }

    public static void main(String[] args) throws IOException,
            FileNotFoundException, XMLStreamException
    {
        String file = "d:\\work\\xml\\test\\valVSrest.xml";
        long[] sizes = { 100000, 200000, 300000, 400000, 500000, 600000 };
        StreamDist dist = new StreamDist(3, false);
        System.out
                .print("  SIZE  |  READING  |  PARSING  |  VALIDATE  |  REST-DIST | RG-DIST ");
        for (int i = 0; i < sizes.length; i++) {
            generateFile(file, sizes[i]);
            double size = ((int) ((((double) ((new File(file)).length() / 1024)) / 1024.0) * 100)) / 100.0;
            System.out.print("\n  " + size + " |");
            Date start = new Date();
            readFile(file);
            Date stop = new Date();
            System.out.print(" " + measure(start, stop) + " |");
            TokenStream ts = new StAXTokenStream(file);
            start = new Date();
            readStream(ts);
            stop = new Date();
            System.out.print(" " + measure(start, stop) + " |");
            ts = new StAXTokenStream(file);
            start = new Date();
            ValidationAutomaton.validate(ts, dtd);
            stop = new Date();
            System.out.print(" " + measure(start, stop) + " |");
            ts = new StAXTokenStream(file);
            start = new Date();
            RestaurationAutomaton.dist(ts, dtd);
            stop = new Date();
            System.out.print(" " + measure(start, stop) + " |");
//            ts = new StAXTokenStream(file);
//            /start = new Date();
//            (new RepairingGraph()).traverse(ts, dtd);
//            stop = new Date();
//            System.out.print(" " + measure(start, stop) + " |");
            //			StAXTokenStream ts1 = new StAXTokenStream(file);
            //			StAXTokenStream ts2 = new StAXTokenStream(file);
            //			
            //            start = new Date();
            //			dist.find(ts1,ts2);
            //			stop = new Date();
            //			System.out.print(" " + measure(start,stop) + " |");
        }
    }

    public static String measure(Date start, Date stop)
    {
        Date diff = new Date(stop.getTime() - start.getTime());
        return "" + (((double) diff.getTime()) / 1000.0);//form.format(diff);
        //return form.format(diff);
    }
}