/*
 * Created on Oct 18, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.test.cqa;

import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import javax.xml.stream.XMLStreamException;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.data.query.Axis;
import edu.buffalo.nsf.xmlcqa.data.query.FilterExpr;
import edu.buffalo.nsf.xmlcqa.data.query.NodeTest;
import edu.buffalo.nsf.xmlcqa.data.query.Query;
import edu.buffalo.nsf.xmlcqa.data.query.Query.Step;
import edu.buffalo.nsf.xmlcqa.data.stream.StAXTokenStream;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenStream;
import edu.buffalo.nsf.xmlcqa.data.stream.generator.DocumentGenerator;
import edu.buffalo.nsf.xmlcqa.data.stream.generator.RandomDocumentGenerator;
import edu.buffalo.nsf.xmlcqa.graph.CQA;
import edu.buffalo.nsf.xmlcqa.test.DTDs;
import edu.buffalo.nsf.xmlcqa.tree.CollectionManager;
import edu.buffalo.nsf.xmlcqa.tree.FreezingCM;
import edu.buffalo.nsf.xmlcqa.tree.OperationManager;
import edu.buffalo.nsf.xmlcqa.tree.OperationManager2;
import edu.buffalo.nsf.xmlcqa.tree.QueryAnswer;
import edu.buffalo.nsf.xmlcqa.tree.StatCM;
import edu.buffalo.nsf.xmlcqa.util.Util;


/**
 * @author staworko
 *
 */
public class RandDocQuery
{
    protected static DecimalFormat format = new DecimalFormat("00.000");
    
    public static String format(double val) {
        return format.format(val);
    }
    
    public static double diff(Date start, Date stop)
    {
        return (stop.getTime() - start.getTime()) / 1000.0;
    }

    public static double measure(double[] timing)
    {
        if (timing.length > 5) {
            Arrays.sort(timing);
            double result = 0.0;
            for (int i = 1; i < timing.length - 1; i++) {
                result += timing[i];
            }
            return result / (timing.length - 2);
        } else {
            double result = 0.0;
            for (int i = 0; i < timing.length; i++) {
                result += timing[i];
            }
            return result / (timing.length);            
        }
    }

    public static void main(String[] args) throws IOException, XMLStreamException
    {
        Query q = new Query(new Query.Step[] {new Step(Axis.CHILD_TR, NodeTest.NODE, FilterExpr.EMPTY), 
                                              new Step(Axis.SELF, NodeTest.TEXT, FilterExpr.createTextFilter("ANS"))});

        DTD dtd = DTDs.dtd[0].init();
        String file = "d:\\work\\xml\\test\\rand.xml";
        
        DocumentGenerator gen = new RandomDocumentGenerator(dtd,Tag.get("a"));
        
        long size[] = { 10000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000,100000,
                       110000,120000,130000,140000,150000,160000,170000,180000,190000,200000,
                       210000,220000,230000,240000,250000,260000,270000,280000,290000,300000
                       };
        
        Map param = new HashMap();
                       
        OperationManager mgr = new OperationManager2(CollectionManager.ARRAY_LIST_CM);
        OperationManager lmgr = new OperationManager2(new FreezingCM(CollectionManager.ARRAY_LIST_CM));
        QueryAnswer qa = new QueryAnswer(mgr);
        CQA cqa = new CQA(lmgr, 10);

        TokenStream ts;
        
        Date start, stop;
        
        System.out.print("  Nodes | Depth | Size | QA | CQA ");
        for (int i = 0; i < size.length; i++) {
            System.out.print("\n  " + size[i] + " | ");
            param.put(DocumentGenerator.SIZE_PARAM, new Long(size[i]));        
            param.put(DocumentGenerator.RATIO_PARAM, new Double(1000.0 / size[i]));        
            gen.generate(file,param);
            double fsize = ((1.0 * new File(file).length()) / 1024.0) / 1024.0;
            System.out.print(" " + Util.measureDepth(file) + " | ");
            System.out.print(" " + format(fsize) + " | ");
            double[] timing = new double[3];
            double tcqa,tqa;
            
            
            //QA
            Arrays.fill(timing, 0.0);
            for (int k = 0; k < timing.length; k++) {
                ts = new StAXTokenStream(file);
                System.gc(); 
                start = new Date();
                qa.compute(ts, q);
                //System.out.print(" (" + s + " )");
                stop = new Date();
                timing[k] = diff(start, stop);
            }
            System.out.print(" " + format(tqa = measure(timing)) + " | ");

            //CQA
            Arrays.fill(timing, 0.0);
            for (int k = 0; k < timing.length; k++) {
                ts = new StAXTokenStream(file);
                System.gc(); 
                start = new Date();
                cqa.compute(ts, q, dtd);
                //System.out.print(" (" + s + " )");
                stop = new Date();
                timing[k] = diff(start, stop);
            }
            System.out.print(" " + format(tcqa = measure(timing)) + " ");            
        
        }
    }
}
