/*
 * Created on Jun 13, 2005
 *
 */
package edu.buffalo.nsf.xmlcqa.test.cqa;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import edu.buffalo.nsf.xmlcqa.data.dtd.Clousure;
import edu.buffalo.nsf.xmlcqa.data.dtd.Concat;
import edu.buffalo.nsf.xmlcqa.data.dtd.DTD;
import edu.buffalo.nsf.xmlcqa.data.dtd.RegExpr;
import edu.buffalo.nsf.xmlcqa.data.dtd.Tag;
import edu.buffalo.nsf.xmlcqa.data.query.Axis;
import edu.buffalo.nsf.xmlcqa.data.query.FilterExpr;
import edu.buffalo.nsf.xmlcqa.data.query.NodeTest;
import edu.buffalo.nsf.xmlcqa.data.query.Query;
import edu.buffalo.nsf.xmlcqa.data.query.Query.Step;
import edu.buffalo.nsf.xmlcqa.data.stream.FileTokenSink;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenSink;
import edu.buffalo.nsf.xmlcqa.data.stream.TokenStream;

/**
 * @author staworko
 *  
 */
public class TestUtil
{

    public static Tag   a = Tag.get("a");
    public static Tag   b = Tag.get("b");
    public static Tag   c = Tag.get("c");
    public static Token A = Token.createTag("a");
    public static Token B = Token.createTag("b");
    public static Token C = Token.createTag("c");
    public static DTD   simpleDTD;
    public static DTD   textDTD;
    static {
        Map defs = new HashMap(3);
        defs.put(a, RegExpr.EPSILON);
        defs.put(b, RegExpr.EPSILON);
        defs.put(c, new Clousure(new Concat(a, b)));
        simpleDTD = new DTD(defs).init();
        defs = new HashMap(3);
        defs.put(a, Tag.PCDATA_TAG);
        defs.put(b, RegExpr.EPSILON);
        defs.put(c, new Clousure(new Concat(a, b)));
        textDTD = new DTD(defs).init();
    }

    public static DTD generateSimpleDTD(int size)
    {
        Map defs = new HashMap(3);
        defs.put(a, RegExpr.EPSILON);
        defs.put(b, RegExpr.EPSILON);
        DTD dtd;
        RegExpr defc = RegExpr.EPSILON;
        for (int j = 0; j < size; j++) {
            defc = new Concat(new Clousure(new Concat(a, b)), defc);
        }
        defs.put(c, defc);
        return new DTD(defs).init();
    }

    public static DTD generateTextDTD(int size)
    {
        Map defs = new HashMap(3);
        defs.put(a, Tag.PCDATA_TAG);
        defs.put(b, RegExpr.EPSILON);
        DTD dtd;
        RegExpr defc = RegExpr.EPSILON;
        for (int j = 0; j < size; j++) {
            defc = new Concat(new Clousure(new Concat(a, b)), defc);
        }
        defs.put(c, defc);
        return new DTD(defs).init();
    }

    public static Query simpleQuery = new Query(new Query.Step[] {
            new Step(Axis.LEFT, NodeTest.NODE, FilterExpr.createTagFilter(a)),
            new Step(Axis.CHILD, NodeTest.STAR, FilterExpr.EMPTY), });
    public static Query textQuery   = new Query(new Query.Step[] {
            new Step(Axis.CHILD, NodeTest.TEXT, FilterExpr
                    .createTextFilter("0")),
            new Step(Axis.CHILD, NodeTest.NODE, FilterExpr.createTagFilter(a)),
            new Step(Axis.SELF, NodeTest.STAR, FilterExpr.EMPTY) });

    public static Query generateSimpleQuery(int length)
    {
        Step[] steps = new Step[length + 2];
        steps[0] = new Step(Axis.LEFT, NodeTest.NODE, FilterExpr
                .createTagFilter(a));
        steps[1] = new Step(Axis.CHILD, NodeTest.STAR, FilterExpr.EMPTY);
        for (int j = 0; j < length; j++) {
            steps[j + 2] = new Step(Axis.CHILD_TR, NodeTest.STAR,
                                    FilterExpr.EMPTY);
        }
        return new Query(steps);
    }

    public static Query generateTextQuery(int length)
    {
        Step[] steps = new Step[length + 3];
        steps[0] = new Step(Axis.CHILD, NodeTest.TEXT, FilterExpr
                .createTextFilter("0"));
        steps[1] = new Step(Axis.CHILD, NodeTest.NODE, FilterExpr
                .createTagFilter(a));
        steps[2] = new Step(Axis.SELF, NodeTest.STAR, FilterExpr.EMPTY);
        for (int j = 0; j < length; j++) {
            steps[j + 3] = new Step(Axis.CHILD_TR, NodeTest.STAR,
                                    FilterExpr.EMPTY);
        }
        return new Query(steps);
    }

    private static DecimalFormat format = new DecimalFormat("00.000");

    public static String format(double val)
    {
        return format.format(val);
    }

    public static double diff(Date start, Date stop)
    {
        return (stop.getTime() - start.getTime()) / 1000.0;
    }

    public static double measure(double[] timing)
    {
        if (timing.length > 5) {
            Arrays.sort(timing);
            double result = 0.0;
            for (int i = 1; i < timing.length - 1; i++) {
                result += timing[i];
            }
            return result / (timing.length - 2);
        } else {
            double result = 0.0;
            for (int i = 0; i < timing.length; i++) {
                result += timing[i];
            }
            return result / (timing.length);            
        }
    }
    
    public static double std_dev(double[] timing)
    {
        double avg = measure(timing);
        if (timing.length > 5) {
            Arrays.sort(timing);
            double sum = 0.0;
            for (int i = 1; i < timing.length - 1; i++) {
                sum += Math.pow(timing[i]-avg,2.0);
            }
            return Math.sqrt(sum / (timing.length - 3));
        } else {
            double sum = 0.0;
            for (int i = 0; i < timing.length; i++) {
                sum += Math.pow(timing[i]-avg,2.0);
            }
            return Math.sqrt(sum / (timing.length - 1));            
        }
        
    }

    public static void generateCleanSimpleFile(String file, int size)
            throws IOException
    {
        TokenSink sink = new FileTokenSink(file);
        sink.consume(C);
        int i = 0;
        while (i < size) {
            sink.consume(A);
            sink.consume(Token.CLOSING_TOKEN);
            sink.consume(B);
            sink.consume(Token.CLOSING_TOKEN);
            i++;
        }
        sink.consume(Token.CLOSING_TOKEN);
        sink.close();
    }

    public static void generateCleanTextFile(String file, int size, int result)
            throws IOException
    {
        TokenSink sink = new FileTokenSink(file);
        double spread = (1.0 * result) / (1.0 * size);
        Random r = new Random();
        sink.consume(C);
        int i = 0;
        while (i < size) {
            sink.consume(A);
            if (r.nextDouble() < spread) {
                sink.consume(Token.createText("0"));
            } else {
                sink.consume(Token.createText("1"));
            }
            sink.consume(Token.CLOSING_TOKEN);
            sink.consume(B);
            sink.consume(Token.CLOSING_TOKEN);
            i++;
        }
        sink.consume(Token.CLOSING_TOKEN);
        sink.close();
    }

    public static void generateDirtySimpleFile(String file, int size, int errors)
            throws IOException
    {
        int threshold = (size / (errors + 1));
        TokenSink sink = new FileTokenSink(file);
        sink.consume(C);
        int i = 0;
        while (i < size) {
            sink.consume(A);
            sink.consume(Token.CLOSING_TOKEN);
            sink.consume(B);
            sink.consume(Token.CLOSING_TOKEN);
            if (++i < size && i % threshold == 0) {
                sink.consume(B);
                sink.consume(Token.CLOSING_TOKEN);
            }
        }
        sink.consume(Token.CLOSING_TOKEN);
        sink.close();
    }

    public static void generateDirtySimpleFile(String file, int size, double errors_ratio)
    throws IOException
    {
        TokenSink sink = new FileTokenSink(file);
        sink.consume(C);
        Random r = new Random();
        for (int i = 0; i < size; i++) {
            sink.consume(A);
            sink.consume(Token.CLOSING_TOKEN);
            if (r.nextDouble() < errors_ratio) {
                //time for an error
                if (r.nextDouble() < 0.5d) {
                    //insertion of A token
                    sink.consume(A);
                    sink.consume(Token.CLOSING_TOKEN);
                    sink.consume(B);
                    sink.consume(Token.CLOSING_TOKEN);                    
                } else {
                    //deletion of B token
                }
            } else {
                sink.consume(B);
                sink.consume(Token.CLOSING_TOKEN);
            }
        }
        sink.consume(Token.CLOSING_TOKEN);
        sink.close();
    }

    public static void generateDirtyTextFile(String file, int size, int result,
                                             int errors) throws IOException
    {
        int threshold = (size / (errors + 1));
        TokenSink sink = new FileTokenSink(file);
        Random r = new Random();
        double spread = (1.0 * size) / (1.0 * result);
        sink.consume(C);
        int i = 0;
        while (i < size) {
            sink.consume(A);
            if (r.nextDouble() < spread) {
                sink.consume(Token.createText("0"));
            } else {
                sink.consume(Token.createText("1"));
            }
            sink.consume(Token.CLOSING_TOKEN);
            sink.consume(B);
            sink.consume(Token.CLOSING_TOKEN);
            if (++i < size && i % threshold == 0) {
                sink.consume(B);
                sink.consume(Token.CLOSING_TOKEN);
            }
        }
        sink.consume(Token.CLOSING_TOKEN);
        sink.close();
    }

    public static void readFile(String file) throws IOException
    {
        Reader r = new BufferedReader(new FileReader(file));
        while (r.read() != -1) {
        }
        r.close();
    }

    public static void readStream(TokenStream ts)
    {
        while (ts.hasNext()) {
            Token t = ts.next();
        }
    }
}