/*
 * Created on Jun 23, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.tree;

import java.util.AbstractCollection;
import java.util.Collection;
import java.util.Iterator;
import edu.buffalo.nsf.xmlcqa.util.Profiler;


/**
 * @author staworko
 *
 */
public class LazyCloneCM extends CMWrapper
{
    public class CollectionConcat extends AbstractCollection
    {
        private Collection inner;
        private Collection active;     
        private int session = 0;
        
        
        /*
         * empty collection
         */
        public CollectionConcat()
        {
            this.inner  = cm.empty();
            this.active = cm.empty();
        }
        
        /*
         * cloning collection
         */
        public CollectionConcat(CollectionConcat col)
        {
            this.inner   = col.inner;
            this.active  = cm.clone(col.active);
            this.session = col.session;
        }
        
//        /*
//         * copying collection 
//         */
//        public CollectionConcat copy(CollectionConcat col)
//        {
//            this.active = cm.copy(this.active,col.active);
//            return this;
//        }
        
        public CollectionConcat cut(CollectionConcat col)
        {
            if (! (this.inner.isEmpty() && col.inner.isEmpty()) && this.inner != col.inner){
                //throw new IllegalArgumentException("Wrong sessions");                
            }
            this.active = cm.cut(this.active, col.active);
            //System.out.println("cut performed: session = " + session + " inner = " +inner.size() + " active = " + this.active.size());
            return this;
        }
        
        public void nextSession()
        {
            Profiler.start("LazyCloneCM.CollectionConcat.nextSession");
            this.inner = cm.add(inner,active);
            this.active = cm.empty();
            this.session++;
            Profiler.stop();
            
        }

        public int size()
        {
            return inner.size() + active.size();
        }

        public Iterator iterator()
        {
            if (inner.isEmpty()) {
                return active.iterator();
            } else {
                return new IteratorConcat(inner,active);
            }
        }
        
        public boolean add(Object o)
        {
            return active.add(o);
        }
        
        public boolean addAll(Collection col)
        {
            return active.addAll(col);
        }
        
//        public String toString()
//        {
//            return "CC:[inner=" + inner + "; active=" + active + "]";
//        }
    }
    
    private class IteratorConcat implements Iterator
    {
        private Iterator inner;
        private Iterator active;
        private Iterator current;
        
        public IteratorConcat(Collection inner, Collection active)
        {
            this.inner = inner.iterator();
            this.active = active.iterator();
            this.current = this.inner;
        }
        
        public void remove()
        {
            current.remove();
        }

        public boolean hasNext()
        {
            return current.hasNext();
        }

        public Object next()
        {
            Object next = current.next();
            if (! current.hasNext()) {
                if (current == inner) {
                    current = active;
                }
            }
            
            return next;
        }
    }
    
    public LazyCloneCM(CollectionManager cm)
    {
        super(cm);
    }

    public Collection empty()
    {
        Profiler.start("LazyConcatCM.empty()");
        return (Collection) Profiler.stop(new CollectionConcat());
    }  
  
    public Collection clone(Collection src)
    {
        Profiler.start("LazyCloneCM.clone()");
        return (Collection) Profiler.stop(new CollectionConcat((CollectionConcat)src));
    }    
    
    public Collection add(Collection dst, Collection src)
    {
        Profiler.start("LazyCloneCM.add()");
        //System.out.print("Adding " + dst + " + " + src + " = ");
        dst.addAll(src);
        //System.out.println(dst);
        Profiler.stop();
        return dst;
    }
    
    public Collection add(Collection dst, TreeNode node)
    {
        Profiler.start("LazyCloneCM.add()");
        //System.out.print("Adding " + dst + " + " + node + " = ");
        dst.add(node);
        //System.out.println(dst);
        Profiler.stop();
        return dst;
    }
    
    public Collection cut(Collection dst, Collection src)
    {
        Profiler.start("LazyCloneCM.cut()");
        return (Collection) Profiler.stop(((CollectionConcat) dst).cut((CollectionConcat) src));
    }
}
