/*
 * Created on Jun 11, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.tree;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import edu.buffalo.nsf.xmlcqa.data.query.Axis;
import edu.buffalo.nsf.xmlcqa.data.query.Query;


/**
 * @author staworko
 *
 */
public class NodeCollector2 implements NodeCollector
{
    protected Query        path;
    protected TreeNode     root;
    protected TreeNode     child;
    protected Collection[] nodes;
    protected Collection[] offspring;
    protected CollectionManager cm;
    
    
    
    public NodeCollector2(CollectionManager cm)
    {
        this.cm = cm;
    }
    
    public NodeCollector2(CollectionManager cm, Query path, TreeNode root)
    {
        this(cm);
        this.path = path;
        nodes = new Collection[path.length];
        
        for (int i = 0; i < path.length; i++) {
            nodes[i] = cm.empty();
        }

        this.root = root;
        
        if (path.step[0].axis.isRefl()) {
            add(0,nodes,root);
        }
    }
    
    private NodeCollector2(NodeCollector2 nc)
    {
        this(nc.cm);
        this.path = nc.path;
        this.root = nc.root;
        this.nodes = new Collection[this.path.length];
        for (int i = 0; i < path.length; i++) {
            this.nodes[i] = cm.clone(nc.nodes[i]);
        }

        if (nc.child != null) {
            this.child = nc.child;
            this.offspring = new Collection[this.path.length];
            for (int i = 0; i < path.length; i++) {
                this.offspring[i] = cm.clone(nc.nodes[i]);
            }
        }
    }
    
    public NodeCollector2(NodeCollector2 nc, TreeNode root)
    {
        this(nc.cm);
        this.path = nc.getPath();
        this.root = root;
        this.nodes = new Collection[path.length];
        
        for (int i = 0; i < path.length; i++) {
            nodes[i] = substitute(nc.getNodes(i),root);
        }
    }
    
    protected  Collection substitute(Collection nodes, TreeNode root)
    {
        if (nodes.isEmpty()) {
            return cm.empty();
        }
        
        Collection result = cm.empty();
        Iterator   iter = nodes.iterator();
        
        while (iter.hasNext()) {
            TreeNode node = (TreeNode) iter.next();
            result.add(node.substitute(root));
        }
        
        return result;
    }
    
    
    public Query getPath()
    {
        return path;
    }

    public TreeNode getRoot()
    {
        return root;
    }

    public Collection getNodes(int i)
    {
        return nodes[i];
    }

    public NodeCollector cloneNC()
    {
        return new NodeCollector2(this);
    }
    
    protected void add(int i, Collection[] ns, TreeNode node) 
    {
        do {
            ns[i] = cm.add(ns[i++],node);
        } while (i < path.length && path.step[i].axis.isRefl());
    }

    protected void add(int i, Collection[] ns,  Collection c)
    {
        do {
            ns[i] = cm.add(ns[i++],c);
        } while (i < path.length && path.step[i].axis.isRefl());
    }
    
    protected void initOffspring(NodeCollector sibling)
    {
        this.child = sibling.getRoot();
        offspring = new Collection[path.length];
        
        
        for (int i = 0; i < path.length; i++){            
            offspring[i] = cm.clone(sibling.getNodes(i));
        }
    }
    
    protected void joinOffspring(NodeCollector sibling)
    {
        for (int i = path.length - 1; i >= 0; i--) {
            offspring[i] = cm.clone(sibling.getNodes(i));
            if (path.step[i].axis.getBaseAxis() == Axis.LEFT &&
                path.step[i].test(child)) {
                if (i > 0) {
                    add(i,offspring,offspring[i-1]);
                } else {
                    add(i,offspring,child);
                }
            }
        }
        
        this.child = sibling.getRoot();
    }
    
    protected void hookNewOffspring()
    {
        for (int i = path.length - 1; i >=0; i--) {
            if (path.step[i].axis.getBaseAxis() == Axis.CHILD &&
                path.step[i].test(child)) {
                if (i > 0) {
                    add(i,nodes,offspring[i-1]);
                } else {
                    add(i,nodes,child);
                }
            }
        }
    }
    
    protected void hookOffspring(NodeCollector2 sibling)    
    {
        if (child == null) {
            initOffspring(sibling);
        } else {
            joinOffspring(sibling);
        }
        
        hookNewOffspring();
    }

    public void dropOffspring()
    {
        this.offspring = null;
        this.child = null;
    }

    public void cut(NodeCollector2 nc)
    {
        if (root != nc.root) {
            throw new IllegalArgumentException("Incompatibe Node Collectors.");
        } 
        
        for (int i = 0; i < path.length; i++) {
            nodes[i] = cm.cut(nodes[i],nc.nodes[i]);            
        }
    }
  
    public String toString()
    {
        return "NC" + Arrays.asList(nodes);
    }
}
