/*
 * Created on Jun 11, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.tree;

import edu.buffalo.nsf.xmlcqa.data.query.Query;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.graph.Edge;
import edu.buffalo.nsf.xmlcqa.graph.Vertex;
import edu.buffalo.nsf.xmlcqa.util.Profiler;


/**
 * @author staworko
 *
 */
public class OperationManager2 implements OperationManager
{

    protected CollectionManager cm;
    
    
    public OperationManager2(CollectionManager cm)
    {
        this.cm = cm;
    }
    
    
    public NodeCollector createRoot(Query path, TreeNode root)
    {
        Profiler.start("OM2.createRoot()");
        return (NodeCollector) Profiler.stop(new NodeCollector2(cm,path,root));
    }

    public NodeCollector createChild(NodeCollector root, int i, Token tok)
    {
        Profiler.start("OM2.createChild()");
        return (NodeCollector) Profiler.stop(new NodeCollector2(cm,root.getPath(),root.getRoot().getChild(i,tok)));
    }

    public NodeCollector hook(NodeCollector root, NodeCollector child)
    {
        Profiler.start("OM2.hook()");
        ((NodeCollector2)root).hookOffspring((NodeCollector2) child);
        return (NodeCollector) Profiler.stop(root);
    }

    public NodeCollector substitute(NodeCollector nc, TreeNode root)
    {
        Profiler.start("OM2.substitute()");
        return (NodeCollector) Profiler.stop(new NodeCollector2((NodeCollector2) nc,root));
    }

    public NodeCollector cut(NodeCollector[] ncs)
    {
        Profiler.start("OM2.cut()");
        NodeCollector2 nc = (NodeCollector2) ncs[0];
        for (int i = 1; i < ncs.length; i++) {
            if (ncs[i] != nc) {
                nc.cut((NodeCollector2) ncs[i]);
            }
        }
        nc.dropOffspring();
        
        return (NodeCollector) Profiler.stop(nc);
    }

    protected boolean uniform(NodeCollector[] ncs)
    {
        Profiler.start("OM2.join()");
        if (ncs.length == 0) {
            Profiler.stop();
            return true;
        } else {
            for (int i = 1; i < ncs.length; i++) {
                if (ncs[i] != ncs[0]) {
                    Profiler.stop();
                    return false;
                }
            }
        }
        
        Profiler.stop();
        return true;
    }
    
    public NodeCollector join(NodeCollector[] ncs, NodeCollector child)
    {
        if (uniform(ncs)) {
            return hook(ncs[0],child);
        }

        Profiler.start("OM2.join()");
        NodeCollector2 nc = (NodeCollector2) hook(ncs[0],child);
        for (int i = 1; i < ncs.length; i++) {
            if (ncs[i] != nc) {
                nc.cut((NodeCollector2) hook(ncs[i],child));
            }
        }
        
        return (NodeCollector) Profiler.stop(nc);
    }

    public NodeCollector clone(NodeCollector nc)
    {
        return ((NodeCollector2) nc).cloneNC();
    }

    public NodeCollector[] clone(NodeCollector[] ncs)
    {
        Profiler.start("OM2.clone()");
        NodeCollector[] res = new NodeCollector[ncs.length];
        
        for (int i = 0; i < res.length; i++) {
            res[i] = clone(ncs[i]);
        }
        
        return (NodeCollector[]) Profiler.stop(res);
    }
    
    public static class ClonningDetected extends Error {

        private static final long serialVersionUID = 5411964641491673978L;};
    
    public void evaluate(Vertex v)
    {
        if (v.isEvaluated()) {
            return;
        }
        
        v = v.domain;
        
        int size = 0;
        Edge e = v.in;
        while (e != null) {
            if (!e.from.isEvaluated()) {
                evaluate(e.from);
            }            
            v.removeFollowingIsoDuplicates(e);
            size += (e.op == Edge.DEL) ? (e.from.getValue().length):(1);
            e = e.nextIn;
        }
        
        NodeCollector[] value = new NodeCollector[size];        
        int i = 0;
            
        e = v.in;
        while (e != null) {
            switch (e.op) {
            case Edge.INS :
            	value[i++] = join(e.from.useValue(this),
            	                e.dtd.getMT(e.from.getValue()[0].getRoot(),
            	                            e.t,
            	                            e.from.getValue()[0].getPath(),
            	                            this)
            	                );
            	break;
            case Edge.REP :
            	value[i++] = join(e.from.useValue(this),e.child);
            	break;
            case Edge.DEL :
                i += copy(value,i,e.from.useValue(this));
        		break;
            }
            e = e.nextIn;
        }
        
            
        while (v != null) {
            v.setValue(value);
            v.dropIncomingEdges();
            v = v.nextDom;
        }
    }
    
    private int copy(Object[] dst, int i, Object[] src)
    {
        System.arraycopy(src,0,dst,i,src.length);

        return src.length;
    }

    public String toString()
    {
        return "";
    }

    public void evaluate(Vertex[] v)
    {
        for (int i = 0; i < v.length; i++) {
            if (v[i].isRegistered()) {
                evaluate(v[i]);
            }
        }
    }


}
