/*
 * Created on Jun 6, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.tree;

import edu.buffalo.nsf.xmlcqa.data.query.Query;
import edu.buffalo.nsf.xmlcqa.data.stream.Token;
import edu.buffalo.nsf.xmlcqa.graph.Vertex;


/**
 * @author staworko
 *
 */
public class StatOM implements OperationManager 
{
    private OperationManager mgr;
    
    public StatOM(OperationManager mgr)
    {
        this.mgr = mgr;
    }
    
    private static final int ROOT  = 0;
    private static final int CHILD = 1;
    private static final int HOOK  = 2;
    private static final int SUBST = 3;
    private static final int CUT   = 4;
    private static final int JOIN  = 5;
    private static final int CLONE = 6;
    private static final int EVAL  = 7;
   // private static final 
    
    private long[] stat = new long[8];
    
    private long[] measure = new long[10];
    private int i = 0;
    private void start()
    {
        measure[i++] = System.currentTimeMillis();
    }
    
    private NodeCollector stop(NodeCollector res, int statNo)
    {
        stop(statNo);
        return res;
    }

    private void stop(int statNo)
    {
        stat[statNo] += System.currentTimeMillis() - measure[--i];
    }

    private NodeCollector[] stop(NodeCollector[] res, int statNo)
    {
        stop(statNo);
        return res;
    }
    
    public void init()
    {
        for (int i = 0; i < stat.length; i++) {
            stat[i] = 0;
        }
    }
    
    public String toString()
    {
        return "Stats[" + 
        		"root  = " + stat[ROOT]/1000.0 + "; " +
        		"child = " + stat[CHILD]/1000.0 + "; " + 
        		"hook  = " + stat[HOOK]/1000.0 + "; " + 
        		"subst = " + stat[SUBST]/1000.0 + "; " + 
        		"cut   = " + stat[CUT]/1000.0 + "; " + 
        		"join  = " + stat[JOIN]/1000.0 + "; " + 
        		"clone = " + stat[CLONE]/1000.0 + "; " +
        		"eval  = " + stat[EVAL]/1000.0 + "; " + 
        		"total = " + 
        				  ( stat[ROOT] + 
        			        stat[CHILD] + 
        	        		stat[HOOK] + 
        	        		stat[SUBST] + 
        	        		stat[CUT] + 
        	        		stat[JOIN] + 
        	        		stat[CLONE] + 
        	        		stat[EVAL]
        			        )/1000.0 +
         		"]";        		
    }

    public NodeCollector createRoot(Query path, TreeNode root)
    {
        start();
        return stop(mgr.createRoot(path,root),ROOT);
    }

    public NodeCollector createChild(NodeCollector root, int i, Token tok)
    {
        start();
        return stop(mgr.createChild(root,i,tok),CHILD);
    }

    public NodeCollector hook(NodeCollector root, NodeCollector child)
    {
        start();
        return stop(mgr.hook(root,child),HOOK);
    }

    
    public NodeCollector substitute(NodeCollector nc, TreeNode root)
    {
        start();
        return stop(mgr.substitute(nc,root),SUBST);
    }

    public NodeCollector cut(NodeCollector[] ncs)
    {
        start();
        return stop(mgr.cut(ncs),CUT);
    }

    
    public NodeCollector join(NodeCollector[] ncs, NodeCollector child)
    {
        start();
        return stop(mgr.join(ncs,child),JOIN);
    }

    public NodeCollector clone(NodeCollector nc)
    {
        start();
        return stop(mgr.clone(nc),CLONE);
    }

    public NodeCollector[] clone(NodeCollector[] ncs)
    {
        start();
        return stop(mgr.clone(ncs),CLONE);
    }

    public void evaluate(Vertex v)
    {
        start();
        mgr.evaluate(v);
        stop(EVAL);
    }

    public void evaluate(Vertex[] v)
    {
        start();
        mgr.evaluate(v);
        stop(EVAL);
    }
}
