/*
 * Created on Jun 26, 2005
 *
  */
package edu.buffalo.nsf.xmlcqa.util;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Stack;
import java.util.TreeMap;


/**
 * @author staworko
 *
 */
public class Profiler
{
    private static boolean active = false;
    private static Stack keyStack;
    private static Map   currentKeys;
    private static Map   stats;
    static {
        init();
    }
    
    public static void init()
    {
        keyStack = new Stack();
        currentKeys = new HashMap();
        stats = new TreeMap();
    }
    
    
    public static void start(Object key)
    {
        if (!active) {
            return;
        }
        if (!currentKeys.containsKey(key)){
            currentKeys.put(key,new Date());
        }
        keyStack.push(key);
            
    }
    
    public static void stop()
    {
        if (!active) {
            return;
        }
        Object key = keyStack.pop();
        long stop = System.currentTimeMillis();
        if (!keyStack.contains(key)) {
            Date start = (Date) currentKeys.remove(key);
            add(key,stop - start.getTime());
        }
    }
    
    public static Object stop(Object o)
    {
        stop();
        return o;
    }

    private static void add(Object key, long inc)
    {
        Long value = (Long) stats.get(key);
        value = (value == null) ? (new Long(inc)):(new Long(value.longValue()+inc));
        stats.put(key,value);
    }
    
    public static void activate()
    {
        active = true;
    }
    
    public static void deactivate()
    {
        active = false;
    }
    
    public static double getStat(Object key)
    {
        if (!active) {
            return 0.0;
        }
        return ((Long)stats.get(key)).longValue()/1000.0;
    }
    
    public static void printStats()
    {
        printStats(stats.keySet().toArray());
    }
    
    public static void printStats(Object[] inspect)
    {
        if (active) {
            int length = findLength(inspect);
            
            System.out.println("Profiling results: ");
            
            for (int i = 0; i < inspect.length; i++) {
                Object key = inspect[i];
                System.out.print(key);
                for (int j = key.toString().length(); j < length; j++) {
                    System.out.print(' ');
                }
                System.out.println(" => " + getStat(key));
            }
        }
        
    }
    
    private static int findLength(Object[] inspect)
    {
        int length = 0;
        for (int i = 0; i < inspect.length; i++) {
            if (length < inspect[i].toString().length()) {
                length = inspect[i].toString().length();
            }
        }
        return length;
    }

}

